/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.*;

/**
 * 在构建命令前对输入数据进行“白名单”机制验证
 *
 * @author WuHaoYang
 * @date 2024/1/10
 */
@Rule(key = "SystemFunctionChecker")
public class SystemFunctionChecker extends IssuableSubscriptionVisitor {

    private static final Set<String> SYSTEM_FUNCTION_NAMES = new HashSet<>(Arrays.asList(
            "exec",
            "getRuntime().exec",
            "ProcessBuilder.command"
    ));

    @Override
    public List<Tree.Kind> nodesToVisit() {
        return Arrays.asList(Tree.Kind.METHOD_INVOCATION, Tree.Kind.NEW_CLASS);
    }

    @Override
    public void visitNode(Tree tree) {
        if (tree.is(Tree.Kind.METHOD_INVOCATION)) {
            MethodInvocationTree methodInvocationTree = (MethodInvocationTree) tree;

            ExpressionTree methodSelect = methodInvocationTree.methodSelect();
            if (methodSelect.is(Tree.Kind.MEMBER_SELECT)) {
                MemberSelectExpressionTree memberSelect = (MemberSelectExpressionTree) methodSelect;
                String methodName = memberSelect.identifier().name();

                if (SYSTEM_FUNCTION_NAMES.contains(methodName)) {
                    checkSystemCommand(methodInvocationTree.arguments(), methodInvocationTree);
                }
            }
        } else if (tree.is(Tree.Kind.NEW_CLASS)) {
            NewClassTree newClassTree = (NewClassTree) tree;
            String className = newClassTree.symbolType().name();

            if (className.equals("ProcessBuilder")) {//判断是否为ProcessBuilder
                checkSystemCommand(newClassTree.arguments(), newClassTree);
            }
        }
    }

    private void checkSystemCommand(List<ExpressionTree> arguments, Tree tree) {
        for (int i = 0; i < arguments.size(); i++) {
            ExpressionTree argument = arguments.get(i);

            // 获取方法的参数
            Tree parentMethod = findEnclosingMethod(tree);
            if (parentMethod != null && parentMethod.is(Tree.Kind.METHOD)) {
                MethodTree methodTree = (MethodTree) parentMethod;
                List<VariableTree> parameters = methodTree.parameters();

                // 检查执行系统命令对象的参数是否与方法的入参相等
                if (i < parameters.size()) {
                    String parameterName = parameters.get(i).simpleName().name();
                    if (argument.is(Tree.Kind.IDENTIFIER) && ((IdentifierTree) argument).name().equals(parameterName)) {
                        System.out.println("参数"+parameterName);
                        reportIssue(tree, "在构建命令前对输入数据进行验证");
                    }
                }
            }
        }
    }

    public void reportIssue(Tree tree, String message) {
        context.reportIssue(this, tree, message);
    }

    private Tree findEnclosingMethod(Tree tree) {
        while (tree != null && !tree.is(Tree.Kind.METHOD)) {
            tree = tree.parent();
        }
        return tree;
    }
}
