/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */

package com.keyware.sonar.java.utils;

import org.sonar.plugins.html.checks.HtmlIssue;
import org.sonar.plugins.html.checks.PreciseHtmlIssue;

import javax.annotation.Nullable;
import java.util.*;

/**
 * Html检查消息验证器
 *
 * @author GuoXin
 * @date 2024/1/20
 */
public final class HtmlCheckMessagesVerifier {

    public static HtmlCheckMessagesVerifier verify(Collection<HtmlIssue> messages) {
        return new HtmlCheckMessagesVerifier(messages);
    }

    private final Iterator<HtmlIssue> iterator;
    private HtmlIssue current;

    private static final Comparator<HtmlIssue> ORDERING = (left, right) -> {
        if (Objects.equals(left.line(), right.line())) {
            return left.message().compareTo(right.message());
        } else if (left.line() == null) {
            return -1;
        } else if (right.line() == null) {
            return 1;
        } else {
            return left.line().compareTo(right.line());
        }
    };

    private HtmlCheckMessagesVerifier(Collection<HtmlIssue> messages) {
        ArrayList<HtmlIssue> messagesList = new ArrayList<>(messages);
        messagesList.sort(ORDERING);
        iterator = messagesList.iterator();
    }

    public HtmlCheckMessagesVerifier next() {
        if (!iterator.hasNext()) {
            throw new AssertionError("\nExpected violation");
        }
        current = iterator.next();
        return this;
    }

    public void noMore() {
        if (iterator.hasNext()) {
            HtmlIssue next = iterator.next();
            throw new AssertionError("\nNo more violations expected\ngot: at line " + next.line());
        }
    }

    private void checkStateOfCurrent() {
        if (current == null) {
            throw new IllegalStateException("Prior to this method you should call next()");
        }
    }

    public HtmlCheckMessagesVerifier atLine(@Nullable Integer expectedLine) {
        checkStateOfCurrent();
        if (!Objects.equals(expectedLine, current.line())) {
            throw new AssertionError("\nExpected: " + expectedLine + "\ngot: " + current.line());
        }
        return this;
    }

    public HtmlCheckMessagesVerifier atLocation(int startLine, int startColumn, int endLine, int endColumn) {
        checkStateOfCurrent();
        PreciseHtmlIssue preciseHtmlIssue = (PreciseHtmlIssue) current;
        if (!Objects.equals(startLine, current.line())) {
            throw new AssertionError("\nExpected: " + startLine + "\ngot: " + current.line());
        }
        if (!Objects.equals(startColumn, preciseHtmlIssue.startColumn())) {
            throw new AssertionError("\nExpected: " + startColumn + "\ngot: " + preciseHtmlIssue.startColumn());
        }
        if (!Objects.equals(endLine, preciseHtmlIssue.endLine())) {
            throw new AssertionError("\nExpected: " + endLine + "\ngot: " + preciseHtmlIssue.endLine());
        }
        if (!Objects.equals(endColumn, preciseHtmlIssue.endColumn())) {
            throw new AssertionError("\nExpected: " + endColumn + "\ngot: " + preciseHtmlIssue.endColumn());
        }
        return this;
    }

    public HtmlCheckMessagesVerifier withMessage(String expectedMessage) {
        checkStateOfCurrent();
        String actual = current.message();
        if (!actual.equals(expectedMessage)) {
            throw new AssertionError("\nExpected: \"" + expectedMessage + "\"\ngot: \"" + actual + "\"");
        }
        return this;
    }

    public HtmlCheckMessagesVerifier withCost(@Nullable Double expectedCost) {
        checkStateOfCurrent();
        if (!Objects.equals(expectedCost, current.cost())) {
            throw new AssertionError("\nExpected: " + expectedCost + "\ngot: " + current.cost());
        }
        return this;
    }

}
